/*
 * Clases para manejar botones en SDL2
 */
#ifndef _SDL_BUTTON
#define _SDL_BUTTON

#define EXPORT __declspec(dllexport)

#include <string>
#include <SDL.h>
#include "sdl_renderer.h"
#include "sdl_texturearray.h"
#include "sdl_font.h"

namespace sdl {
/** \class Button
 *  \brief Encapsula un botón
 *
 * Un botón es un rectángulo que muestra una imagen y un texto que
 * desencadena una acción cuando el usuario hace clic sobre él con el mouse.
 *
 */

class Button {
protected:
    Renderer& renderer;     /**< Contexto de renderización */
    TextureArray* texture;  /**< Textura con gráficos de botones */
    int index;              /**< Índice del gráfico dentro de la textura */
    Font& font;             /**< Fuente de caracteres */
    std::string line;       /**< Texto del botón */
    int textsize;           /**< Tamaño del texto */
    Color color;            /**< Color del texto */
    Rect rect;              /**< Área activa del botón */
    bool active;            /**< Indica si el botón es pulsable */
    bool visible;           /**< Indica si el botón debe ser mostrado */
    Point posTexture;       /**< Posición de la textura dentro del botón */
    Point posText;          /**< Posición del texto dentro del botón */
    int margen;             /**< Pixels de separación entre el texto y gráfico y borde */

    /** \brief Verifica si un punto está dentro del área del botón
     *
     * \param p Point Referencia al punto a verificar
     * \return bool Devuelve true si el punto está dentro del área del botón
     *
     */
    bool TestPoint(Point& p) { return SDL_PointInRect(p.Get(), rect.Get()); }
public:
    /** \brief Constructor
     *
     * \param r Renderer& Referencia a objeto Renderer que indica el contexto de renderizado de la ventana que contiene el botón
     * \param pos Point Posición del botón
     * \param t TextureArray* Array de texturas que contiene el gráfico a mostrar en el botón
     * \param i int Índice del gráfico dentro del objeto TextureArray
     * \param str std::string Texto del botón
     * \param f Font& Referencia a la fuente a usar para mostrar el texto
     * \param ts int Tamaño del texto
     * \param c Color Color del texto
     *
     */
    EXPORT Button(Renderer& r, Point pos, TextureArray* t, int i, std::string str, Font& f, int ts, Color c);
    /** \brief Modifica la posición del botón
     *
     * \param pos Point Nueva posición del botón
     *
     */
    EXPORT void SetPos(Point &pos) { rect.X() = pos.X(); rect.Y() = pos.Y(); }
    /** \brief Muestra el botón en pantalla
     *
     */
    EXPORT void Render();
    /** \brief Permite activar o desactivar el botón. Un botón desactivado no puede ser pulsado.
     *
     * \param a bool Si es true, activa el botón
     *
     */
    EXPORT void SetActive(bool a) { active=a; if(active) visible=true; }
    /** \brief Permite ocultar o mostrar el botón
     *
     * \param v bool Si es true el botón será visible.
     *
     */
    EXPORT void SetVisible(bool v) { visible=v; }
    /** \brief Indica si el botón ha sido pulsado
     *
     * \param p Point Punto donde se ha hecho clic con el ratón
     * \return EXPORT bool Devuelve true si el punto está dentro del área del botón
     *
     */
    EXPORT bool Pushed(Point p);
};

} // namespace

#endif // _SDL_BUTTON
