/*
 * Clases para manejar Chunk de SDL_mixer
 * Mix_LoadWAV
 * Mix_FreeChunk
 * Mix_PlayChannel
 */

#ifndef _SDL_CHUNK
#define _SDL_CHUNK

#include <SDL.h>
#include <SDL_mixer.h>

namespace sdl {

/** \class Chunk
 *  \brief Encapsula un sonido
 */
class Chunk {
protected:
    Mix_Chunk *chunk;   /**< Estructura encapsulada */
    int channel;        /**< Canal de audio */

public:
    /** \brief Constructor por defecto
     *
     */
    Chunk() { chunk=NULL; }
    /** \brief Constructor
     *
     * \param file const char* Fichero que contiene el sonido
     *
     */
    Chunk(const char *file) {
        chunk = Mix_LoadWAV(file);
    }
    /** \brief Destructor
     *
     * Libera los recursos asociados al chunk
     *
     */
    ~Chunk() {
        Mix_FreeChunk(chunk);
    }
    /** \brief Reproduce el sonido
     *
     * \param ch int Canal a utilizar, -1 para elegir uno libre
     * \param loops int Nmero de repeticiones, 0 para repetir indefinidamente
     * \return int Devuelve el nmero del canal utilizado
     *
     */
    int Play(int ch=-1, int loops=0) {
        channel = Mix_PlayChannel(ch, chunk, loops);
        return channel;
    }
    /** \brief Valor del volumen maestro a establecer para todos los canales
     *
     * \param volumen int Valor del volumen a establecer
     * \return int Volumen establecido
     *
     */
    int MasterVolumen(int volumen) {
        return Mix_MasterVolume(volumen);
    }
    /** \brief Establece el volumen para la reproduccin de sonidos en un canal
     *
     * \param volumen int Valor del volumen a establecer
     * \return int Volumen establecido
     *
     */
    int Volumen(int volumen) {
        return Mix_Volume(channel, volumen);
    }
    /** \brief Detiene la reproduccin de un sonido en el canal actual
     *
     */
    void Pausa() {
        Mix_Pause(channel);
    }
    /** \brief Restablece la reproduccin del sonido en el canal actual
     *
     */
    void Resume() {
        Mix_Resume(channel);
    }
    /** \brief Averigua si el sonido del canal actual est pausado
     *
     * \return bool true si el sonido est pausado
     *
     */
    bool Pausado() {
        return Mix_Paused(channel);
    }
    /** \brief Detiene el sonido en el canal actual
     *
     * \return int 0 si tiene xito, -1 en caso de error
     *
     */
    int Halt() {
        return Mix_HaltChannel(channel);
    }
    /** \brief Reproduce un nuevo objeto musical, incrementando el volumen del audio.
     *
     * \param loops int El nmero de veces que sonido debe reproducirse en bucle, -1 para indefinidamente
     * \param ms int El nmero de milisegundos de fundido de entrada.
     * \return int Devuelve cero en caso de xito, -1 en caso de error.
     *
     */
    int FadeIn(int loops=0, int ms=1000) {
        return Mix_FadeInChannel(channel, chunk, loops, ms);
    }
    /** \brief Reproduce un fragmento de audio en el canal actual, incrementando el volumen del audio, durante un tiempo mximo.
     *
     * \param loops int El nmero de veces que sonido debe reproducirse en bucle, -1 para indefinidamente
     * \param ms int El nmero de milisegundos de fundido de entrada.
     * \param ticks int El nmero mximo de milisegundos de este chunk a mezclar para la reproduccin.
     * \return int Devuelve qu canal se utiliz para reproducir el sonido, o -1 si no se pudo reproducir el sonido.
     *
     */
    int FadeInTimed(int loops=0, int ms=1000, int ticks=-1) {
        return Mix_FadeInChannelTimed(channel, chunk, loops, ms, ticks);
    }
    /** \brief Detiene el canal actual despus de desvanecerlo durante un tiempo especificado.
     *
     * \param ms int Nmero de milisegundos de fundido antes de detener el canal.
     * \return int Devuelve el nmero de canales programados para desvanecerse.
     *
     */
    int FadeOut(int ms) {
        return Mix_FadeOutChannel(channel, ms);
    }
    /** \brief Consulta el estado de desvanecimiento del canal actual
     *
     * \return Mix_Fading Devuelve el estado actual de desvanecimiento del canal. MIX_NO_FADING, MIX_FADING_OUT o MIX_FADING_IN
     *
     */
    Mix_Fading Fading() {
        return Mix_FadingChannel(channel);
    }
    /** \brief Establece la posicin espacial de un canal
     *
     * \param angle Sint16 ngulo, en grados. El Norte es 0, y va en el sentido de las agujas del reloj.
     * \param distance Uint8 Distancia; 0 es el oyente, 255 es la distancia mxima.
     * \return Devuelve cero si hay error
     *
     */

    int SetPosition(Sint16 angle, Uint8 distance) {
        return Mix_SetPosition(channel, angle, distance);
    }
};
} // namespace

#endif // _SDL_CHUNK

