/*
 * Clases para manejar circulos en SDL2
 */
#ifndef _SDL_CIRCLE
#define _SDL_CIRCLE

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <cmath>
#include "sdl_point.h"

namespace sdl {

/** \class Circle
 *  \brief Encapsula una circunferencia con coordenadas enteras
 *
 * Se usa en hitboxes para la detección de colisiones
 *
 */
class Circle {
protected:
    Point center;   /**< Centro */
    int radius;     /**< Radiu */
public:
    /** \brief Constructor a partir de coordenadas enteras y radio
     *
     * \param x int Coordenada x del centro
     * \param y int Coordenada y del centro
     * \param r int Radio
     *
     */
    EXPORT Circle(int x=0, int y=0, int r=0) : center(x, y), radius(r) {}
    /** \brief Constructor a partir de un punto para el centro y radio
     *
     * \param c Point Coordenadas del centro
     * \param r int Radio
     *
     */
    EXPORT Circle(Point &c, int r=0) : center(c), radius(r) {}
    /** \brief Devuelve las coordenadas del centro de la circunferencia
     *
     * \return Point& referencia al valor del centro de la circunferencia
     *
     */
    EXPORT Point &Center() { return center; }
    /** \brief Obtiene la coordenada x del centro
     *
     * \return int coordenada x del centro
     *
     */
    EXPORT int &X() { return center.X(); }
    /** \brief Obtiene la coordenada y del centro
     *
     * \return int coordenada y del centro
     *
     */
    EXPORT int &Y() { return center.Y(); }
    /** \brief Obtiene el radio
     *
     * \return int valor del radio
     *
     */
    EXPORT int &R() { return radius; }
};

/** \class FCircle
 *  \brief Encapsula una circunferencia con coordenadas en punto flotante
 *
 * Se usa en hitboxes para la detección de colisiones
 *
 */
class FCircle {
protected:
    FPoint center;  /**< Centro */
    float radius;   /**< Radio */
public:
    /** \brief Constructor a partir de coordenadas enteras y radio
     *
     * \param x float Coordenada x del centro
     * \param y flaot Coordenada y del centro
     * \param r float Radio
     *
     */
    EXPORT FCircle(float x=0, float y=0, float r=0) : center(x, y), radius(r) {}
    /** \brief Constructor a partir de un punto para el centro y radio
     *
     * \param c FPoint Coordenadas del centro
     * \param r float Radio
     *
     */
    EXPORT FCircle(FPoint &c, float r=0) : center(c), radius(r) {}
    /** \brief Mueve la circunferencia rotando en ángulo dado y a la distancia especificada, aplicando una escala
     *
     *  Se usan las formulas de traslación para calcular las nuevas coordenadas del centro según:
     *  x' = x * x" * cos(α) + y * y" * sin(α) + dx
     *  y' = -x * x" * sin(α) + y * y" * cos(α) + dy
     * \param alfa float ángulo de rotación
     * \param displacement FPoint desplazamiento
     * \param escale FPoint escala en los ejes x e y
     * \return FCircle objeto con el resultado de la traslación
     *
     */
    EXPORT FCircle Move(float alfa, FPoint &displacement, FPoint &escale);
    /** \brief Devuelve las coordenadas del centro de la circunferencia
     *
     * \return FPoint& referencia al valor del centro de la circunferencia
     *
     */
    EXPORT FPoint &Center() { return center; }
    /** \brief Obtiene la coordenada x del centro
     *
     * \return float coordenada x del centro
     *
     */
    EXPORT float &X() { return center.X(); }
    /** \brief Obtiene la coordenada y del centro
     *
     * \return float coordenada y del centro
     *
     */
    EXPORT float &Y() { return center.Y(); }
    /** \brief Obtiene el radio
     *
     * \return float valor del radio
     *
     */
    EXPORT float &R() { return radius; }
};

} // namespace

#endif // _SDL_CIRCLE

