/*
 * Clases para manejar un color en SDL2
 */
#ifndef _SDL_COLOR
#define _SDL_COLOR

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <string>

namespace sdl {

/** \class Color
 *  \brief Encapsula un color
 *
 */
class Color {
private:
    SDL_Color color;    /**< Color SDL */

public:
    /** \brief Constructor a partir de componentes RGBA en enteros sin signo de 8 bits
     *
     * \param r Uint8 Componente roja
     * \param g Uint8 Componente verde
     * \param b Uint8 Componete azul
     * \param a Uint8 Componente alfa
     *
     */
    Color(Uint8 r=0, Uint8 g=0, Uint8 b=0, Uint8 a=255) {
        color.r=r;
        color.g=g;
        color.b=b;
        color.a=a;
    }
    /** \brief Constructor a partir de una cadena
     *
     * Admite formatos de cadena "#RRGGBBAA", "RRGGBBAA", "#RRGGBB", "RRGGBB", "#RGBA", "#RGB", "RGBA", "RGB"
     * \param c string cadena que define el color
     *
     */
    EXPORT Color(const std::string c); // Formato #RRGGBBAA, RRGGBBAA, #RRGGBB, RRGGBB, #RGBA, #RGB, RGBA, RGB
    /** \brief Operador de tipo que convierte un objeto Color a SDL_Color
     *
     * \return SDL_Color retorna un objeto SDL_Color
     *
     */
    EXPORT operator SDL_Color() { return color; }
    /** \brief Obtiene la componente roja
     *
     * \return Uint8 valor de la componente roja
     *
     */
    EXPORT Uint8 GetR() const { return color.r; }
    /** \brief Obtiene la componente verde
     *
     * \return Uint8 valor de la componente verde
     *
     */
    EXPORT Uint8 GetG() const { return color.g; }
    /** \brief Obtiene la componente azul
     *
     * \return Uint8 valor de la componente azul
     *
     */
    EXPORT Uint8 GetB() const { return color.b; }
    /** \brief Obtiene la componente alfa
     *
     * \return Uint8 valor de la componente alfa
     *
     */
    EXPORT Uint8 GetA() const { return color.a; }
    /** \brief Asigna un nuevo valor a la componente roja
     *
     * \param r Uint8 nuevo valor de la componente roja
     *
     */
    EXPORT void SetR(Uint8 r) { color.r = r; }
    /** \brief Asigna un nuevo valor a la componente verde
     *
     * \param g Uint8 nuevo valor de la componente verde
     *
     */
    EXPORT void SetG(Uint8 g) { color.g = g; }
    /** \brief Asigna un nuevo valor a la componente azul
     *
     * \param b Uint8 nuevo valor de la componente azul
     *
     */
    EXPORT void SetB(Uint8 b) { color.b = b; }
    /** \brief Asigna un nuevo valor a la componente alfa
     *
     * \param a Uint8 nuevo valor de la componente alfa
     *
     */
    EXPORT void SetA(Uint8 a) { color.a = a; }
};

} // namespace

#endif // _SDL_COLOR

