/*
 * Clases para manejar un créditos en SDL2
 */
#ifndef _SDL_CREDITS
#define _SDL_CREDITS

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <string>
#include <vector>
#include "sdl_game.h"
#include "sdl_rect.h"
#include "sdl_font.h"

namespace sdl {

/** \class CreditsLine
 *  \brief Clase para manejar una línea de texto de créditos
 *
 */
class CreditsLine {
protected:
    Rect area;              /**< Área para activación con mouse */
    Font &font;             /**< Fuente */
    int textsize;           /**< Tamaño de fuente */
    std::string line;       /**< Línea de texto completa de la opción */
    /** \brief Cambia la posición de la línea de texto
     *
     * \param p Point& Nueva posición
     *
     */
    void Move(Point& p);

public:
    /** \brief Constructor
     *
     * \param f Fonnt& fuente a utilizar para mostrar el texto
     * \param c strint texto de la línea
     * \param size int tamaño de la fuente
     *
     */
    EXPORT CreditsLine(Font& f, const std::string c, int size) : font(f), textsize(size), line(c) {}
    /** \brief Muestra la línea en pantalla
     *
     * \param renderer Renderer& referencia al contexto de renderizado
     * \param pos Point& coordenadas del texto
     * \param fg Color color del texto
     *
     */
    EXPORT void Render(Renderer& renderer, Point& pos, Color fg) {
        font.RenderUTF8_Blended(line, pos, fg);
    }
};

/** \class Credits
 *  \brief Clase para manejar una pantalla de créditos
 *
 */
class Credits : public Game{
protected:
    Font font;                       /**< Fuente */
    int textsize;                    /**< Tamaño de fuente para opciones */
    std::string title;               /**< Texto del título */
    Point titlePos;                  /**< Coordenadas del título */
    Point textPos;                   /**< Coordenadas de la primera línea de menú */
    std::vector<CreditsLine> line;   /**< Opciones de menú */
    Color titleColor;                /**< Color del texto de título */
    Color textColor;                 /**< Color del texto de opciones */
    int retval;                      /**< Valor de retorno, aunque no es realmente interesante */

public:
    /** \brief Constructor
     *
     * \param win Window& referencia a la ventana para mostrar los créditos
     * \param ren Renderer& referencia a contexto de renderizado
     * \param f string& nombre del fichero con la fuente a utilizar
     *
     */
    EXPORT Credits(Window& win, Renderer& ren, const std::string& f) : Game(win, ren),
        font(ren, f), title(""), titleColor("#ffff00ff"), textColor("#ffff00ff"), retval(0) {}
    /** \brief Establece la posición del título
     *
     * \param x int coordenada x de la posición del título
     * \param y int coordenada y de la posición del título
     *
     */
    EXPORT void SetTitlePos(int x, int y) { titlePos = Point(x, y); }
    /** \brief Establece la posición del título
     *
     * \param p Point coordenadas de la posición del título
     *
     */
    EXPORT void SetTitlePos(sdl::Point p) { titlePos = Point(p); }
    /** \brief Establece la posición de la primera línea de texto
     *
     * \param x int coordenada x de la posición de la primera línea de texto
     * \param y int coordenada y de la posición de la primera línea de texto
     *
     */
    EXPORT void SetTextPos(int x, int y) { textPos = Point(x, y); }
    /** \brief Establece la posición de la primera línea de texto
     *
     * \param p Point coordenadas de la posición de la primera línea de texto
     *
     */
    EXPORT void SetTextPos(sdl::Point p) { textPos = Point(p); }
    /** \brief Asigna el texto del título
     *
     * \param t string& nuevo texto del título
     *
     */
    EXPORT void SetTitle(const std::string& t) { title = t; }
    /** \brief Asigna el color para el texto del título
     *
     * \param tc string& nuevo color para el texto del título
     *
     */
    EXPORT void SetTitleColor(const Color& tc) { titleColor = tc; }
    /** \brief Asigna el color para el texto de las líneas
     *
     * \param tc string& nuevo color para el texto de las líneas
     *
     */
    EXPORT void SetTextColor(const Color& tc) { textColor = tc; }
    /** \brief Añade una línea de texto de créditos
     *
     * \param txt string& texto de la línea
     * \param size tamaño del texto
     *
     */
    EXPORT void AddLine(const std::string& txt, int size=17);
    /** \brief Borra el contenido de la ventana usando el color indicado
     *
     * \param color Color color para el fondo de la ventana
     *
     */
    EXPORT void Clear(Color color=Color("#000000ff"));
    /** \brief Obtiene una referencia a la fuente usada para el texto
     *
     * \return Font& referencia a la fuente
     *
     */
    EXPORT Font& GetFont() { return font; }
    /** \brief Asigna el tamaño de la fuente a utilizar
     *
     * \param pt int tamaño de la fuente
     *
     */
    EXPORT void SetTextSize(int pt);
    /** \brief Inicializa el objeto
     *
     */
    EXPORT void Init() { Clear(); }
    /** \brief Actualiza el contenido de la ventana
     *
     * AL tratarse de un método virtual es necesario definirlo, aunque en este casco no hace nada
     *
     */
    EXPORT void Update() {} /**< No es necesario */
    /** \brief Procesa los eventos que lleguen a la aplicación mientras se muestran los créditos
     *
     */
    EXPORT void Events() {
        SDL_Keysym key;

        while(event.Poll()) {
            if(event.Type() == SDL_KEYUP) {
                key = event.Key().keysym;
                if(key.sym == SDLK_ESCAPE) { retval= -1; }
            }
            else if(event.Type() == SDL_MOUSEBUTTONDOWN) { retval= -1; }
        }
    }
    /** \brief Muestra el contenido de la ventana de créditos
     *
     */
    EXPORT void Render() {
        font.SetOutline(1);
        font.SetSize(55);
        font.RenderUTF8_Blended(title, titlePos, titleColor);

        font.SetOutline(0);
        font.SetSize(textsize);
        for(size_t i=0; i<line.size(); i++) {
            line[i].Render(renderer, textPos, textColor);
            textPos.Y()+=textsize+10;
        }
        SDL_RenderPresent(renderer);
    }
    /** \brief Ejecuta el bucle de juego de la ventana de créditos
     *
     */

    EXPORT char Run() {
        Init();
        Render();
        do {
            Events();
        } while(!retval);
        return retval;
    }
};

} // namespace sdl
#endif // _SDL_CREDITS


