/*
 * Clases para manejar fuentes en SDL2
 *
 */

#ifndef _SDL_FONT
#define _SDL_FONT

#define EXPORT __declspec(dllexport)

#include <string>
#include <SDL.h>
#include <SDL_ttf.h>
#include "sdl_rect.h"
#include "sdl_renderer.h"
#include "sdl_color.h"

namespace sdl {

/** \class Font
 *  \brief Clase para encapsular una fuente de caracteres SDL
 *
 */
class Font {
protected:
    TTF_Font *font;     /**< Estructura encapsulada */
    Renderer& renderer; /**< Contexto de renderizado para mostrar texto */

public:
    /** \brief Constructor
     *
     * \param render Renderer& Contexto de renderizado necesario para mostrar texto en pantalla
     * \param fontfile string& Nombre del fichero que contiene la fuente
     * \param ptsize int Tamaño de la fuente
     *
     */
    EXPORT Font(Renderer& render, const std::string& fontfile, int ptsize=17) : renderer(render)
        { font = TTF_OpenFont(fontfile.c_str(), ptsize); }
    /** \brief Destructor
     *
     * Libera la memoria usada por la fuente
     *
     */
    EXPORT ~Font() { TTF_CloseFont(font); }
    /** \brief Establece el estilo de la fuente
     *
     * Permite establecer uno o varios estilos combinados con OR entre los disponibles:
     * TTF_STYLE_NORMAL, TTF_STYLE_BOLD, TTF_STYLE_ITALIC, TTF_STYLE_UNDERLINE y TTF_STYLE_STRIKETHROUGH
     *
     * \param style int nuevo estilo para aplicar a la fuente
     *
     */
    EXPORT void SetStyle(int style) { TTF_SetFontStyle(font, style); }
    /** \brief Establece el tamaño de la fuente
     *
     * \param ptsize int nuevo tamaño de la fuente
     *
     */
    EXPORT void SetSize(int ptsize) { TTF_SetFontSize(font, ptsize); }
    /** \brief Establece el contorno actual de una fuente.
     *
     * \param outline int un valor de contorno positivo
     *
     */
    EXPORT void SetOutline(int outline) { TTF_SetFontOutline(font, outline); }
    /** \brief Establece la dirección que se utilizará para dar forma al texto
     *
     * \param direction TTF_Direction Uno de los valores TTF_Direction para la nueva dirección para el texto:
     * TTF_DIRECTION_LTR (De izquierda a derecha).
     * TTF_DIRECTION_RTL (De derecha a izquierda).
     * TTF_DIRECTION_TTB (De arriba a abajo).
     * TTF_DIRECTION_BTT (de abajo arriba).
     *
     */
    EXPORT void SetDirection(TTF_Direction direction) { TTF_SetFontDirection(font, direction); }
    /** \brief Establece la configuración del interior actual
     *
     * \param hinting int El ajuste de interior es un valor único:
     * TTF_HINTING_NORMAL
     * TTF_HINTING_LIGHT
     * TTF_HINTING_MONO
     * TTF_HINTING_NONE
     * TTF_HINTING_LIGHT_SUBPIXEL (disponible en SDL_ttf 2.0.18 y posteriores)
     *
     */
    EXPORT void SetHinting(int hinting) { TTF_SetFontHinting(font, hinting); }
    /** \brief Obtiene el área rectangular necesaria para mostrar el texto indicado
     *
     * \param text string& texto para el que se calcula el área
     * \return Rect objeto con las dimensiones del área
     *
     */
    EXPORT Rect GetArea(const std::string& text);
    /** \brief Muestra un texto en pantalla
     *
     * \param text string& texto a mostrar
     * \param x int coordenada x
     * \param y int coordenada y
     * \param fg Color color utilizado para el texto
     *
     */
    EXPORT void RenderUTF8_Blended(const std::string& text, int x, int y, Color fg);
    /** \brief Muestra un texto en pantalla
     *
     * \param text string& texto a mostrar
     * \param p Point& coordenadas
     * \param fg Color color utilizado para el texto
     *
     */
    EXPORT void RenderUTF8_Blended(const std::string& text, Point& p, Color fg) {
        RenderUTF8_Blended(text, p.X(), p.Y(), fg);
    }
};

// TODO: crear variantes SetNormal, SetBold.. UnsetNormal, UnsetBold ??
// TODO: crear métodos SetLTR, SetRTL, SetTTB y SetBTT

} // namespace
#endif // _SDL_FONT
