/*
 * Clases para manejar juegos en SDL2
 */
#ifndef _SDL_GAME
#define _SDL_GAME

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <string>
#include <cstdlib>
#include <ctime>

#include "sdl_window.h"
#include "sdl_renderer.h"
#include "sdl_event.h"

namespace sdl {

/** \fn random
 *  \brief Obtiene un nmero aleatorio entero en el rango de 0 al indicado
 *
 * \param n int valor del lmite superior del rango
 * \return EXPORT int nmero aleatorio entero
 *
 */
EXPORT int random(int n);
/** \fn randomf
 *  \brief Obtiene un nmero aleatorio float en el rango de 0 al indicado
 *
 * \param n float valor del lmite superior del rango
 * \return EXPORT float nmero aleatorio float
 *
 */
EXPORT float randomf(float n);

/** \class Game
 *  \brief Clase base virtual pura para encapsular un juego
 *
 * Esta clase se usar para crear juegos, pero tambin mens, ventanas de crditos, etc
 * En general, cualquier cosa que requiera un bucle de juego: init, events, update, render.
 *
 */
class Game {
protected:
    Window& window;         /**< Referencia de la ventana del juego */
    Renderer& renderer;     /**< Referencia a contexto de renderizado */
    Uint64 tick, tick0;     /**< Valores necesarios para el clculo de delta time */
    Event event;            /**< Objeto para el tratamiento de eventos */
    bool quit;              /**< Verdadero si hay que salir del bucle de juego */

public:
    /** \brief Constructor
     *
     * \param win Window& Referencia a ventana del juego
     * \param ren Renderer& Referencia a contexto de renderizado
     * \param w int anchura de la ventana
     * \param h int altura de la ventana
     *
     */
    EXPORT Game(Window &win, Renderer &ren, int w=800, int h=600) : window(win), renderer(ren), quit(false) {
        window.SetSize(w, h);
        window.Center();
        std::srand(std::time(NULL));
    }
    /** \brief Obtiene una referencia de la ventana
     *
     * \return Window& Valor de retorno
     *
     */
    EXPORT Window &getWindow() { return window; }
    /** \brief Obtiene una referencia al contexto de renderizado
     *
     * \return Renderer& Valor de retorno
     *
     */
    EXPORT Renderer &getRenderer() { return renderer; }
    /** \brief Obtiene el valor del miembro quit, true si se debe abandonar el bucle de juego
     *
     * \return bool true si se debe abandonar el bucle de juego
     *
     */
    EXPORT bool Quit() { return quit; }
    /** \brief Mtodo virtual puro para la inicializacin del juego
     *
     * Establece los valores iniciales para los objetos del juego
     */
    EXPORT virtual void Init() {}
    /** \brief Mtodo virtual puro para el procesamiento de eventos del juego
     *
     * Procesa los eventos necesarios para manejar el juego
     */
    EXPORT virtual void Events() {}
    /** \brief Mtodo virtual puro para la actualizacin de los objetos del juego
     *
     * Actualiza los objetos del juego en funcin de los eventos producidos desde la iteracin anterior del bucle de juego
     *
     */
    EXPORT virtual void Update() {}
    /** \brief Mtodo virtual puro para actualizar la ventana
     *
     */
    EXPORT virtual void Render() {}
    /** \brief Mtodo virtual que contiene el bucle del juego
     *
     * No debera ser necesario modificarlo como norma general, pero queda a discrecin del programador y de las necesidades del juego
     *
     * \return virtual char valor de retorno, necesario en clases derivadas, como Menu
     *
     */
    EXPORT virtual char Run() {
        Init();
        tick0 = SDL_GetTicks64();
        do {
            tick = SDL_GetTicks64()-tick0;
            tick0 = SDL_GetTicks64();
            Events();
            Update();
            Render();
            SDL_Delay(1);
        } while(!Quit());
        return ' ';
    }
};

} // namespace sdl
#endif // _SDL_GAME
