/*
 * Clases para manejar hitboxes en SDL2
 */
#ifndef _SDL_HITBOX
#define _SDL_HITBOX

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <vector>
#include "sdl_point.h"
#include "sdl_rect.h"
#include "sdl_circle.h"

namespace sdl {

/** \enum shapeType
 *  \brief Enumerado para tipos de formas
 */
enum shapeType {
    rectangle=0,
    circle
};

/** \class Shape
 *  \brief Clase para encapsular formas
 *
 * Esta clase se usar para crear hitboxes necesarias para el clculo de colisiones
 *
 */
class Shape {
protected:
    shapeType type;     /**< Tipo de forma */
    union {             /**< Parmetros de la forma, una unin de los objetos que pueden definir una forma */
        FRect re;       /**< Objeto para una forma rectangular */
        FCircle ci;     /**< Objeto para una forma circular */
    };
public:
    /** \brief Constructor por defecto
     *
     * Por defecto se crea una forma rectangular nula
     *
     */
    EXPORT Shape() : type(rectangle), re({0,0,0,0}) {}
    /** \brief Constructor para forma circular
     *
     * \param c FCircle datos de la circunferencia para definir la forma
     *
     */
    EXPORT Shape(FCircle c) : type(circle), ci(c) {}
    /** \brief Constructor para forma rectangular
     *
     * \param r FRect datos del rectngulo para definir la forma
     *
     */
    EXPORT Shape(FRect r) : type(rectangle), re(r) {}
    /** \brief Mueve la forma rotando en ngulo dado y a la distancia especificada, aplicando una escala
     *
     * \param alfa float ngulo de rotacin
     * \param displacement FPoint& desplazamiento
     * \param escale FPoint& escala
     * \return EXPORT Shape forma desplazada
     *
     */
    EXPORT Shape Move(float alfa, FPoint &displacement, FPoint &escale);
    /** \brief Obtiene el objeto circunferencia asociada a la forma
     *
     * \return FCircle& referencia a la circunferencia
     *
     */
    EXPORT FCircle &Ci() { return ci; }
    /** \brief Obtiene el objeto rectngulo asociado a la forma
     *
     * \return FCircle& referencia al rectngulo
     *
     */
    EXPORT FRect &Re() { return re; }
    /** \brief Amistad con la clase Hitbox para que tenga acceso a los miembros privados de Shape
     */
    friend class Hitbox;
};

/** \class Hitbox
 *  \brief Clase para encapsular hitboxes
 *
 * Esta clase se usar para el clculo de colisiones
 *
 */
class Hitbox {
protected:
    std::vector<Shape> hitbox;                              /**< Formas de hitbox normalizadas */
    std::vector<Shape> hitboxT;                             /**< Formas de la hitbox trasladada */
    /** \brief Mtodo privado para averiguar si hay colisin entre dos crculos
     *
     * \param c1 FCircle& primer crculo
     * \param c2 FCircle& segundo crculo
     * \return bool true si colisionan
     *
     */
    bool CollisionCircleCircle(FCircle &c1, FCircle &c2);
    /** \brief Mtodo privado para averiguar si hay colisin entre dos rectngulos
     *
     * \param r1 FRect& primer rectngulo
     * \param r2 FRect& segundo rectngulo
     * \return bool true si colisionan
     *
     */
    bool CollisionRectRect(FRect &r1, FRect &r2);
    /** \brief Mtodo privado para averiguar si hay colisin entre un crculo y un rectngulo
     *
     * \param r1 FRect& rectngulo
     * \param c1 FCircle& crculo
     * \return bool true si colisionan
     *
     */
    bool CollisionRectCircle(FRect &r1, FCircle &c1);

public:
    /** \brief Constructor
     *
     */
    EXPORT Hitbox() {}
    /** \brief Redimensiona el vector de formas trasladadas (privado?)
     *
     * Redimensiona el vector de formas trasladadas para que coincida con el de las formas del hitbox
     *
     */
    EXPORT void ResizeT() { hitboxT.resize(hitbox.size()); }
    /** \brief Aade una forma al hitbox
     *
     * \param f Shape nueva forma para aadir
     *
     */
    EXPORT void InsertShape(const Shape &f) { hitbox.push_back(f); }
    /** \brief Averigua si la hitbox actual colisiona con otra
     *
     * \param h Hitbox& Segunda hitbox para el clculo
     * \return bool true si ambas hitboxes colisionan
     *
     */
    EXPORT bool Collision(Hitbox& h);
    /** \brief Mueve la hitbox rotando en ngulo dado y a la distancia especificada, aplicando una escala
     *
     * \param alfa float ngulo de rotacin
     * \param displacement FPoint& desplazamiento
     * \param escale FPoint& escala
     *
     */
    EXPORT void Move(float alfa, FPoint &displacement, FPoint &escale);
    /** \brief Elimina las formas que componen la hitbox
     *
     */
    EXPORT void Reset() {
        hitbox.clear();
    }
    /** \brief Obtiene la forma que define el hitbox con el ndice dado
     *
     * \param i int ndice dentro del vector de formas
     * \return EXPORT Shape valor de retorno
     *
     */
    EXPORT Shape GetShape(int i) { return hitboxT[i]; }
};

} // namespace sdl

#endif // _SDL_HITBOX
