/*
 * Clases para manejar un menu en SDL2
 */
#ifndef _SDL_MENU
#define _SDL_MENU

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <string>
#include <vector>
#include "sdl_game.h"
#include "sdl_rect.h"
#include "sdl_font.h"

namespace sdl {

/** \class MenuOption
 *  \brief Clase para manejar una línea de menú
 *
 */
class MenuOption {
protected:
    Rect area;              /**< Área para activación con mouse */
    Font &font;             /**< Fuente */
    int textsize;           /**< Tamaño de fuente */
    char key;               /**< Tecla de activación para la opción*/
    std::string line;       /**< Línea de texto completa de la opción */
    std::string cad;        /**< Texto de la opción, excluyendo tecla de activación y marca de selección */
    bool selected;          /**< Opción seleccionada/activa */
    /** \brief Cambia la posición de la línea de texto
     *
     * Calcula el texto total de la línea, añadiendo la marca de selección si es necesario y la tecla de activación.
     * también calcula los valores del área en función de la nueva posición
     *
     * \param p Point& Nueva posición
     *
     */
    void Move(Point& p);

public:
    /** \brief Constructor
     *
     * \param f Fonnt& fuente a utilizar para mostrar el texto
     * \param c strint texto de la línea
     * \param k char tecla para activar la opción del menú
     * \param size int tamaño de la fuente
     *
     */
    EXPORT MenuOption(Font& f, const std::string c, char k, int size) : font(f), textsize(size), key(k), cad(c), selected(false) {}
    /** \brief Obtiene el carácter que activa la opción de menú
     *
     * \return char tecla de activación
     *
     */
    EXPORT char Key() { return key; }
    /** \brief Establece el estado de activación de la opción de menú
     *
     * \param s bool nuevo valor de activación
     *
     */
    EXPORT void SetSelected(bool s=true) { selected = s; }
    /** \brief Verifica si las coordenadas indicadas están dentro del área que ocupa el texto de la opción
     *
     * /param p Point& punto a comprobar
     * \return bool verdadero si el punto está dentro del área
     *
     */
    EXPORT bool TestPoint(Point &p) { return SDL_PointInRect(p.Get(), area.Get()); }
    /** \brief Muestra la línea en pantalla
     *
     * \param renderer Renderer& referencia al contexto de renderizado
     * \param pos Point& coordenadas del texto
     * \param fg Color color del texto
     *
     */
    EXPORT void Render(Renderer& renderer, Point& pos, Color fg);
};

/** \class Menu
 *  \brief Clase para manejar una pantalla de menú
 *
 */
class Menu : public Game{
protected:
    Font font;                       /**< Fuente */
    int textsize;                    /**< Tamaño de fuente para opciones */
    std::string title;               /**< Texto del título */
    Point titlePos;                  /**< Coordenadas del título */
    Point menuPos;                   /**< Coordenadas de la primera línea de menú */
    std::vector<MenuOption> options; /**< Opciones de menú */
    Color titleColor;                /**< Color del texto de título */
    Color optionColor;               /**< Color del texto de opciones */
    char retval;                     /**< Valor de retorno, corresponde a la opción elegida*/

public:
    /** \brief Constructor
     *
     * \param win Window& referencia a la ventana para mostrar los créditos
     * \param ren Renderer& referencia a contexto de renderizado
     * \param f string& nombre del fichero con la fuente a utilizar
     *
     */
    EXPORT Menu(Window& win, Renderer& ren, const std::string& f) : Game(win, ren),
        font(ren, f), title(""), titleColor("#ffff00ff"), optionColor("#ffff00ff"), retval(0) {}
    /** \brief Establece la posición del título
     *
     * \param x int coordenada x de la posición del título
     * \param y int coordenada y de la posición del título
     *
     */
    EXPORT void SetTitlePos(int x, int y) { titlePos = Point(x, y); }
    /** \brief Establece la posición del título
     *
     * \param p Point coordenadas de la posición del título
     *
     */
    EXPORT void SetTitlePos(sdl::Point p) { titlePos = Point(p); }
    /** \brief Establece la posición de la primera línea de texto
     *
     * \param x int coordenada x de la posición de la primera línea de texto
     * \param y int coordenada y de la posición de la primera línea de texto
     *
     */
    EXPORT void SetMenuPos(int x, int y) { menuPos = Point(x, y); }
    /** \brief Establece la posición de la primera línea de texto
     *
     * \param p Point coordenadas de la posición de la primera línea de texto
     *
     */
    EXPORT void SetMenuPos(sdl::Point p) { menuPos = Point(p); }
    /** \brief Asigna el texto del título
     *
     * \param t string& nuevo texto del título
     *
     */
    EXPORT void SetTitle(const std::string& t) { title = t; }
    /** \brief Asigna el color para el texto del título
     *
     * \param bk string& nuevo color para el texto del título
     *
     */
    EXPORT void SetTitleColor(const Color& bk) { titleColor = bk; }
    /** \brief Asigna el color para el texto de las opciones
     *
     * \param fg string& nuevo color para el texto de las opciones
     *
     */
    EXPORT void SetOptionColor(const Color& fg) { optionColor = fg; }
    /** \brief Añade una opción de menú
     *
     * \param opt string& texto de la opción de menu
     * \param key char tecla para activar la opción
     * \param size tamaño del texto
     *
     */
    EXPORT void AddOption(const std::string& opt, char key, int size=17);
    /** \brief Selecciona la opción indicada y deselecciona la opción previamente activa
     *
     * \param k char tecla de activación de la opción a activar
     *
     */
    EXPORT void SetSel(char k);
    /** \brief Borra el contenido de la ventana usando el color indicado
     *
     * \param color Color color para el fondo de la ventana
     *
     */
    EXPORT void Clear(Color color=Color("#000000ff"));
    /** \brief Obtiene una referencia a la fuente usada para el texto
     *
     * \return Font& referencia a la fuente
     *
     */
    EXPORT Font& GetFont() { return font; }
    /** \brief Asigna el tamaño de la fuente a utilizar
     *
     * \param pt int tamaño de la fuente
     *
     */
    EXPORT void SetTextSize(int pt);
    /** \brief Inicializa el objeto Menu
     *
     */
    EXPORT void Init() { Clear(); }
    /** \brief Actualiza el contenido de la ventana
     *
     * AL tratarse de un método virtual es necesario definirlo, aunque en este casco no hace nada
     *
     */
    EXPORT void Update() {} /**< No es necesario */
    /** \brief Procesa los eventos que lleguen a la aplicación mientras se muestran los créditos
     *
     */
    EXPORT void Events() {
        SDL_Keysym key;

        while(event.Poll()) {
            if(event.Type() == SDL_KEYUP) {
                key = event.Key().keysym;
                for(size_t i=0; !retval && i<options.size(); i++) {
                    if(key.sym == options[i].Key()) retval = key.sym;
                }
                if(key.sym == SDLK_ESCAPE) { retval= -1; }
            }
            else if(event.Type() == SDL_MOUSEBUTTONDOWN) {
                sdl::Point point(event.Button().x, event.Button().y);
                retval = -1;
                for(size_t i=0; i<options.size(); i++)
                    if(options[i].TestPoint(point)) retval=options[i].Key();
            }
        }
    }
    /** \brief Muestra el contenido de la ventana de créditos
     *
     */
    EXPORT void Render() {
        font.SetOutline(1);
        font.SetSize(55);
        font.RenderUTF8_Blended(title, titlePos, titleColor);

        font.SetOutline(0);
        font.SetSize(textsize);
        for(size_t i=0; i<options.size(); i++) {
            options[i].Render(renderer, menuPos, optionColor);
            menuPos.Y()+=textsize+10;
        }
        SDL_RenderPresent(renderer);
    }
    EXPORT char Run() {
        Init();
        Render();
        do {
            Events();
        } while(!retval);
        return retval;
    }
};

} // namespace sdl
#endif // _SDL_MENU

