/*
 * Clases para manejar música SDL_mixer
 * Mix_LoadMUS
 * Mix_FreeMusic
 * Mix_PlayMusic
 */

#ifndef _SDL_MUSIC
#define _SDL_MUSIC

#include <SDL.h>
#include <SDL_mixer.h>

namespace sdl {

/** \class Music
 *  \brief Clase para cargar y reproducir archivos de audio con música
 *
 */
class Music {
protected:
    Mix_Music *music;       /**< El formato interno de un fragmento de música interpretado a través de códecs */

public:
    /** \brief Constructor por defecto
     */
    Music() { music=NULL; }
    /** \brief Constructor que carga un archivo de audio
     *
     * \param file char* ruta del fichero a cargar
     *
     */
    Music(const char *file) {
        music = Mix_LoadMUS(file);
    }
    /** \brief Destructor
     *
     * Libera la memoria asociada al fichero de música.
     */
    ~Music() {
        Mix_FreeMusic(music);
    }
    /** \brief Carga un fichero de audio
     *
     * Si ya se había cargado un fichero previamente, libera el objeto antes de cargar el nuevo.
     *
     * \param file char* ruta del fichero a cargar
     *
     */
    void Load(const char *file) {
        if(music) Mix_FreeMusic(music);
        music = Mix_LoadMUS(file);
    }
    /** \brief Reproduce el audio
     *
     * \param loops int número de veces que se reproducirá. 0 para rsproducir una vez.
     *
     */
    void Play(int loops) {
        Mix_PlayMusic(music, loops);
    }
    /** \brief Establece el volumen de reproducción.
     *
     * \param volume int valor del volumen entre 0 y MIX_MAX_VOLUME (128)
     * \return int
     *
     */
    int Volume(int volume) {
        return Mix_VolumeMusic(volume);
    }
    /** \brief Detiene la reproducción de la música.
     *
     */
    void Pause() {
        Mix_PauseMusic();
    }
    /** \brief Reanuda la reproducción de la música previamente pausada.
     *
     */
    void Resume() {
        Mix_ResumeMusic();
    }
    /** \brief Consulta si un canal determinado está en pausa.
     *
     * \return bool verdadero si la reproducción está pausada
     *
     */
    bool Paused() {
        return Mix_PausedMusic();
    }
    /** \brief Detiene la reproducción de la música.
     *
     * Esta función es la complementaria de Play, y no equivale a Pause.
     */
    void Halt() {
        Mix_HaltMusic();
    }
    /** \brief Reproduce un nuevo objeto musical, incrementando el volumen del audio.
     *
     * \param loops int El número de veces que la música debe reproducirse en bucle, -1 para reproducirlo (no realmente) infinitamente.
     * \param ms int El número de milisegundos de fundido de entrada.
     * \return int 0 si tiene éxito, -1 en caso de error
     *
     */
    int FadeIn(int loops=0, int ms=1000) {
        return Mix_FadeInMusic(music, loops, ms);
    }
    /** \brief Reproduce un nuevo objeto musical, incrementando el volumen del audio, desde una posición inicial.
     *
     * \param loops int El número de veces que la música debe reproducirse en bucle, -1 para reproducirlo (no realmente) infinitamente.
     * \param ms int El número de milisegundos de fundido de entrada.
     * \param pos double La posición inicial dentro de la música, en segundos, donde debe comenzar la reproducción.
     * \return int 0 si tiene éxito, -1 en caso de error
     *
     */
    int FadeInPos(int loops=0, int ms=1000, double pos=0.0) {
        return Mix_FadeInMusicPos(music, loops, ms, pos);
    }
    /** \brief Detiene el flujo de música después de desvanecerlo durante un tiempo especificado.
     *
     * \param ms int Número de milisegundos de fundido antes de detener el canal.
     * \return int Devuelve un valor distinto de cero si la música estaba programada para desvanecerse, cero en caso contrario. Si no se está reproduciendo música, devuelve cero.
     *
     */
    int FadeOut(int ms) {
        return Mix_FadeOutMusic(ms);
    }
    /** \brief Consulta el estado de desvanecimiento del flujo musical.
     *
     * \return Mix_Fading Devuelve el estado actual de desvanecimiento del flujo musical: MIX_NO_FADING, MIX_FADING_OUT o MIX_FADING_IN
     *
     */
    Mix_Fading Fading() {
        return Mix_FadingMusic();
    }
};
} // namespace

#endif // _SDL_MUSIC
