/*
 * Clases para manejar puntos en SDL2
 * SDL_Point
 * SDL_FPoint
 */
#ifndef _SDL_POINT
#define _SDL_POINT

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <cmath>

namespace sdl {

/** \class Point
 *  \brief Clase para manipular puntos con coordenadas enteras
 *
 */
class Point {
protected:
    SDL_Point point;    /**< Una estructura que define un punto en dos dimensiones. */
public:
    /** \brief Constructor
     *
     * \param x int coordenada x
     * \param y int coordenada y
     *
     */
    EXPORT Point(int x=0, int y=0) {
        point.x = x;
        point.y = y;
    }
    /** \brief Constructor a partir de un SDL_Point
     *
     * \param p SDL_Point& estructura con punto
     *
     */
    EXPORT Point(SDL_Point &p) : point(p) {}
    /** \brief Obtiene un puntero a la estructura SDL_Point
     *
     * \return SDL_Point* valor de retorno
     *
     */
    EXPORT SDL_Point *Get() { return &point; }
    /** \brief Operador de conversin de tipo
     *
     * \return SDL_Point conviente un objeto Point a SDL_Point
     *
     */
    EXPORT operator SDL_Point() const { return point; }
    /** \brief Obtiene una referencia a la coordenada x del punto
     *
     * \return int& referencia a coordenada x
     *
     */
    EXPORT int &X() { return point.x; }
    /** \brief Obtiene una referencia a la coordenada y del punto
     *
     * \return int& referencia a coordenada y
     *
     */
    EXPORT int &Y() { return point.y; }
};

/** \class FPoint
 *  \brief Clase para manipular puntos con coordenadas float
 *
 */
class FPoint {
protected:
    SDL_FPoint point;    /**< Una estructura que define un punto en dos dimensiones con valores en coma flotante. */
public:
    /** \brief Constructor
     *
     * \param x float coordenada x
     * \param y float coordenada y
     *
     */
    EXPORT FPoint(float x=0.0, float y=0.0) {
        point.x = x;
        point.y = y;
    }
    /** \brief Constructor a partir de un SDL_FPoint
     *
     * \param p SDL_FPoint& estructura con punto
     *
     */
    EXPORT FPoint(SDL_FPoint &p) : point(p) {}
    /** \brief Obtiene un puntero a la estructura SDL_FPoint
     *
     * \return SDL_FPoint* valor de retorno
     *
     */
    EXPORT SDL_FPoint &Get() { return point; }
    /** \brief Operador de conversin de tipo
     *
     * \return SDL_FPoint conviente un objeto FPoint a SDL_FPoint
     *
     */
    EXPORT operator SDL_FPoint() { return point; }
    /** \brief Mueve el punto rotando en ngulo dado y a la distancia especificada, aplicando una escala
     *
     * \param alfa float ngulo de rotacin
     * \param displacement FPoint& desplazamiento
     * \param  escale FPoint& escala
     * \return FPoint punto trasladado
     *
     */
    EXPORT FPoint Move(float alfa, const FPoint &displacement, const FPoint &escale) const;
    /** \brief Obtiene una referencia a la coordenada x del punto
     *
     * \return float& referencia a coordenada x
     *
     */
    EXPORT float &X() { return point.x; }
    /** \brief Obtiene una referencia a la coordenada y del punto
     *
     * \return float& referencia a coordenada y
     *
     */
    EXPORT float &Y() { return point.y; }
};

} // namespace

#endif // _SDL_POINT
