/*
 * Clases para manejar rectángulos en SDL2
 * SDL_Rect
 * SDL_FRect
 */
#ifndef _SDL_RECT
#define _SDL_RECT

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <cmath>
#include "sdl_point.h"

namespace sdl {

/** \class Rect
 *  \brief Clase para manipular rectángulos con coordenadas enteras
 *
 */
class Rect {
protected:
    SDL_Rect rect;      /**< Una estructura que contiene la definición de un rectángulo, con el origen en la esquina superior izquierda. */
public:
    /** \brief Constructor a partir de valores enteros
     *
     * \param x int valor de coordenada x de esquina superior izquierda
     * \param y int valor de coordenada y de esquina superior izquierda
     * \param w int anchura
     * \param h int altura
     *
     */
    EXPORT Rect(int x=0, int y=0, int w=0, int h=0) {
        rect.x = x;
        rect.y = y;
        rect.w = w;
        rect.h = h;
    }
    /** \brief Constructor a partir de estructura SDL_Rect
     *
     * \param r SDL_Rect estructura con datos del rectángulo
     *
     */
    EXPORT Rect(SDL_Rect r) : rect(r) {}
    /** \brief Obtiene un puntero a la estructura SDL_Rect
     *
     * \return SDL_Rect* valor de retorno
     *
     */
    EXPORT SDL_Rect *Get() { return &rect; }
    /** \brief Operador de conversión de tipo
     *
     * \return SDL_Rect conviente un objeto Rect a SDL_Rect
     *
     */
    EXPORT operator SDL_Rect() { return rect; }
    /** \brief Obtiene una referencia a la coordenada x de la esquina superior izquierda
     *
     * \return int& referencia a coordenada x
     *
     */
    EXPORT int &X() { return rect.x; }
    /** \brief Obtiene una referencia a la coordenada y de la esquina superior izquierda
     *
     * \return int& referencia a coordenada y
     *
     */
    EXPORT int &Y() { return rect.y; }
    /** \brief Obtiene una referencia a la anchura del rectángulo
     *
     * \return int& referencia a anchura
     *
     */
    EXPORT int &W() { return rect.w; }
    /** \brief Obtiene una referencia a la altura del rectángulo
     *
     * \return int& referencia a altura
     *
     */
    EXPORT int &H() { return rect.h; }
};

/** \class FRect
 *  \brief Clase para manipular rectángulos con coordenadas en coma flotante
 *
 */
class FRect {
protected:
    SDL_FRect rect;      /**< Una estructura que contiene la definición de un rectángulo, con el origen en la esquina superior izquierda. */
public:
    /** \brief Constructor a partir de valores en coma flotante
     *
     * \param x float valor de coordenada x de esquina superior izquierda
     * \param y float valor de coordenada y de esquina superior izquierda
     * \param w float anchura
     * \param h float altura
     *
     */
    EXPORT FRect(float x=0.0, float y=0.0, float w=0.0, float h=0.0) {
        rect.x = x;
        rect.y = y;
        rect.w = w;
        rect.h = h;
    }
    /** \brief Constructor a partir de estructura SDL_FRect
     *
     * \param r SDL_FRect estructura con datos del rectángulo
     *
     */
    EXPORT FRect(SDL_FRect &r) : rect(r) {}
    /** \brief Obtiene un puntero a la estructura SDL_FRect
     *
     * \return SDL_FRect* valor de retorno
     *
     */
    EXPORT SDL_FRect *Get() { return &rect; }
    /** \brief Operador de conversión de tipo
     *
     * \return SDL_FRect conviente un objeto FRect a SDL_FRect
     *
     */
    EXPORT operator SDL_FRect() { return rect; }
    /** \brief Obtiene las coordenadas de la esquina superior izquierda del rectángulo
     *
     * \return FPoint Objeto con el punto correspondiente a la esquina superior izquierda
     *
     */
    EXPORT FPoint Corner() { return FPoint(rect.x, rect.y); }
    /** \brief Mueve el rectángulo rotando en ángulo dado y a la distancia especificada, aplicando una escala
     *
     * \param alfa float ángulo de rotación
     * \param displacement FPoint& desplazamiento
     * \param  escale FPoint& escala
     * \return FRect rectángulo trasladado
     *
     */
    EXPORT FRect Move(float alfa, FPoint &displacement, FPoint &escale);
    /** \brief Obtiene una referencia a la coordenada x de la esquina superior izquierda
     *
     * \return float& referencia a coordenada x
     *
     */
    EXPORT float &X() { return rect.x; }
    /** \brief Obtiene una referencia a la coordenada y de la esquina superior izquierda
     *
     * \return float& referencia a coordenada y
     *
     */
    EXPORT float &Y() { return rect.y; }
    /** \brief Obtiene una referencia a la anchura del rectángulo
     *
     * \return float& referencia a anchura
     *
     */
    EXPORT float &W() { return rect.w; }
    /** \brief Obtiene una referencia a la altura del rectángulo
     *
     * \return float& referencia a altura
     *
     */
    EXPORT float &H() { return rect.h; }
};

} // namespace

#endif // _SDL_RECT
