/*
 * Clases para manejar renderers en SDL2
 * SDL_CreateRenderer
 * SDL_DestroyRenderer
 * SDL_SetRenderDrawColor
 * SDL_RenderClear
 * SDL_RenderDrawLineF
 * SDL_RenderDrawPointF
 * SDL_RenderPresent
 */
#ifndef _SDL_RENDERER
#define _SDL_RENDERER

#define EXPORT __declspec(dllexport)

#include <vector>
#include <SDL.h>
#include "sdl_window.h"
#include "sdl_color.h"
#include "sdl_point.h"
#include "sdl_rect.h"

namespace sdl {

/** \class Renderer
 *  \brief Clase para manipular un contexto de renderizado
 *
 */
class Renderer {
protected:
    SDL_Renderer *renderer;     /**< Estructura opaca que representa el estado del renderizado. */

public:
    /** \brief Constructor por defecto
     *
     */
    EXPORT Renderer() : renderer(NULL) {}
    /** \brief Constructor
     *
     * \param window Window& ventana a la que se asociar el contexto de renderizado
     * \param flags SDL_RendererFlags 0, uno o ms SDL_RendererFlags combinados con OR.
     */
    EXPORT Renderer(Window &window, SDL_RendererFlags flags) {
        renderer = SDL_CreateRenderer(window, -1, flags);
    }
    /** \brief Destructor
     *
     * Destruye el contexto de renderizado de una ventana y libera las texturas asociadas.
     *
     */
    EXPORT ~Renderer() {
        SDL_DestroyRenderer(renderer);
    }
    /** \brief Devuelve el contexto de renderizado SDL
     *
     */
    EXPORT operator SDL_Renderer*() { return renderer; }
    /** \brief Establece el color para las operaciones de trazado de puntos y figuras
     *
     * \param r Uint8 componente roja
     * \param g Uint8 componente verde
     * \param b Uint8 componente azul
     * \param a Uint8 componente alfa
     *
     */
    EXPORT void SetDrawColor(Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
        SDL_SetRenderDrawColor(renderer, r, g, b, a);
    }
    /** \brief Establece el color para las operaciones de trazado de puntos y figuras
     *
     * \param color Color& objeto de color
     *
     */
    EXPORT void SetDrawColor(const Color& color) {
        SDL_SetRenderDrawColor(renderer, color.GetR(), color.GetG(), color.GetB(), color.GetA());
    }
    /** \brief Borra el contenido del contexto de renderizado
     *
     */
    EXPORT void Clear() {
        SDL_RenderClear(renderer);
    }
    /** \brief Dibuja las lneas contenidas en un vector de puntos
     *
     * TODO: Usar SDL_RenderDrawLinesF(SDL_Renderer*, const SDL_FPoints*, int);
     *
     * \param puntos std::vector<FPoint>
     *
     */
    EXPORT void DrawLinesF( std::vector<FPoint>& puntos) const {
//        SDL_RenderDrawLinesF(renderer, &puntos[0], puntos.size());
        for(size_t i=0; i<puntos.size()-1; i++) {
            SDL_RenderDrawLineF(renderer, puntos[i].X(), puntos[i].Y(), puntos[i+1].X(), puntos[i+1].Y());
        }
    }
    /** \brief Dibuja un punto
     *
     * \param punto FPoint punto a trazar
     *
     */
    EXPORT void DrawFPoint(FPoint punto) {
        SDL_RenderDrawPointF(renderer, punto.X(), punto.Y());
    }
    /** \brief Dibuja un punto
     *
     * \param punto Point punto a trazar
     *
     */
    EXPORT void DrawPoint(Point punto) {
        SDL_RenderDrawPoint(renderer, punto.X(), punto.Y());
    }
    /** \brief Dibuja un rectngulo
     *
     * \param re FRect rectngulo a trazar
     *
     */
    EXPORT void DrawFRect(FRect re) {
        SDL_RenderDrawRectF(renderer, re.Get());
    }
    /** \brief Dibuja un rectngulo
     *
     * \param re Rect rectngulo a trazar
     *
     */
    EXPORT void DrawRect(Rect re) {
        SDL_RenderDrawRect(renderer, re.Get());
    }
    /** \brief Dibuja un rectngulo relleno
     *
     * \param re FRect rectngulo a trazar
     *
     */
    EXPORT void FillFRect(FRect re) {
        SDL_RenderFillRectF(renderer, re.Get());
    }
    /** \brief Dibuja un rectngulo relleno
     *
     * \param re Rect rectngulo a trazar
     *
     */
    EXPORT void FillRect(Rect re) {
        SDL_RenderFillRect(renderer, re.Get());
    }
    /** \brief Actualiza la pantalla con cualquier renderizacin realizada desde la llamada anterior.
     *
     */
    EXPORT void Present() {
        SDL_RenderPresent(renderer);
    }
};

} // namespace

#endif // _SDL_RENDERER
