/*
 * Clases para manejar arrays de texturas en SDL2
 */
#ifndef _SDL_TEXTURE_ARRAY
#define _SDL_TEXTURE_ARRAY

#define EXPORT __declspec(dllexport)

#include <vector>
#include <string>
#include <SDL.h>
#include <SDL_image.h>
#include "sdl_renderer.h"

namespace sdl {

/** \class TextureArray
 * \brief Clase que encapsula una única textura compuesta
 *
 * La textura contiene un array de subtexturas del mismo tamaño que pueden ser mostradas individualmente
 */
class TextureArray {
protected:
    Renderer& renderer;     /**< Contexto de renderizado */
    SDL_Texture* texture;   /**< Textura */
    std::vector<Rect> rect; /**< Rectángulos contenedores de subtexturas */
    int w, h;               /**< Anchura y altura de cada subtextura */
    int rows, cols;         /**< Número de filas y columnas */

public:
    /** \brief Constructor
     *
     * \param render Renderer& contexto de renderizado
     * \param file string ruta del fichero que contiene el gráfico de la textura
     * \param tileWidth int anchura de cada subtextura
     * \param tileHeight int altura de cada subtextura
     *
     */
    EXPORT TextureArray(Renderer& render, const std::string file, int tileWidth=0, int tileHeight=0);
    /** \brief Destructor
     *
     * Destruye la textura especificada.
     *
     */
    EXPORT ~TextureArray() {
        SDL_DestroyTexture(texture);
    }
    /** \brief Obtiene la anchura de las subtexturas
     *
     * \return int valor de la anchura de las subtexturas
     *
     */
    EXPORT int W() { return w; }
    /** \brief Obtiene la altura de las subtexturas
     *
     * \return int valor de la altura de las subtexturas
     *
     */
    EXPORT int H() { return h; }
    /** \brief Obtiene un puntero a la estructura SDL_Texture
     *
     */
    SDL_Texture* Get() { return texture; }
    /** \brief Obtiene un puntero a la estructura SDL_Texture
     *
     */
    EXPORT operator SDL_Texture*() { return texture; }
    /** \brief Traslada la subtextura al punto indicado
     *
     * \param punto Point punto de destino
     * \param index int índice de la subtextura
     *
     */
    EXPORT void Draw(Point punto, int index) {
        Rect dest=Rect(punto.X(), punto.Y(), w, h);
        SDL_RenderCopy(renderer, texture, rect[index].Get(), dest.Get());
    }
};

} // namespace

#endif // _SDL_TEXTURE_ARRAY
