/*
 * Clases para manejar hilos
 */

#ifndef _SDL_THREAD
#define _SDL_THREAD

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <cstdio>

namespace sdl {

/** \class Thread
 * \brief Clase que encapsula un hilo
 *
 */
class Thread {
protected:
    SDL_ThreadFunction tfunction;       /**< Puntero a funcin a invocar en el hilo */
    void* data;                         /**< Datos a pasar a la funcin del hilo */
    SDL_Thread *handle;                 /**< Manipulador del hilo */
    int retval;                         /**< Valor de retorno del hilo */
    bool detach;                        /**< Indica si la funcin est desconectada del hilo */
public:
    /** \brief Constructor por defecto
     *
     */
    EXPORT Thread() : tfunction(NULL), data(NULL), handle(NULL), detach(false) {}
    /** \brief Constructor
     *
     * \param funct SDL_ThreadFunction Funcin asociada al hilo
     * \param name char* Nombre del hilo
     * \param dat void*  Datos a pasar a la funcin del hilo
     * \param stack size_t Tamao de la pila asociada al hilo
     *
     */
    EXPORT Thread(SDL_ThreadFunction funct, const char* name, void* dat, const size_t stack=0) : tfunction(funct), data(dat), detach(false) {
        handle = SDL_CreateThreadWithStackSize(tfunction, name, stack, data);
    }
    /** \brief Destructor
     *
     * Espera a que la funcin del hilo retorne antes de destruir el objeto.
     *
     */
    EXPORT ~Thread() {
        if(!detach) SDL_WaitThread(handle, &retval);
    }
    /** \brief Asigna una nueva funcin al hilo.
     *
     * \param funct SDL_ThreadFunction Puntero a la funcin
     *
     */
    EXPORT void SetFunction(SDL_ThreadFunction funct) { tfunction=funct; }
    /** \brief Asigna un bloque de datos para pasar a la funcin del hilo.
     *
     * \param dat void* Nuevo bloque de datos
     *
     */
    EXPORT void SetData(void *dat) { data = dat; }
    /** \brief Inicia el hilo
     *
     * \param name const char* Nombre del hilo
     * \param stack const size_t Tamao de la pila asociada al hilo
     *
     */
    EXPORT void Init(const char* name, const size_t stack=0) {
        if(tfunction != NULL) handle = SDL_CreateThreadWithStackSize(tfunction, name, stack, data);
    }
    /** \brief Espera a que la funcin asociada al hilo termine,
     *
     * \return int Valor de retorno.
     *
     */
    EXPORT int Wait() {
        if(!detach) {
            SDL_WaitThread(handle, &retval);
        } else {
            retval=-1;
        }
        return retval;
    }
    /** \brief Deja que un hilo se limpie al salir sin intervencin.
     *
     */
    EXPORT void Detach() {
        SDL_DetachThread(handle);
        detach = true;
    }
    /** \brief Obtiene el nombre asociado al hilo
     *
     * \return char* Nombre del hilo
     *
     */
    EXPORT const char* GetName() const {
        if(!detach) return SDL_GetThreadName(handle);
        return NULL;
    }
    /** \brief Obtiene el identificador del hilo.
     *
     * \return SDL_threadID Identificado
     *
     */
    EXPORT SDL_threadID GetId() {
        if(!detach) return SDL_GetThreadID(handle);
        return 0;
    }
};

} // namespace

#endif // _SDL_THREAD

