/*
 * Clases para manejar temporizadores
 */

#ifndef _SDL_TIMER
#define _SDL_TIMER

#define EXPORT __declspec(dllexport)

#include <SDL.h>

namespace sdl {

/** \class Timer
 * \brief Clase que encapsula un temporizador.
 *
 */
class Timer {
protected:
    SDL_TimerCallback CallBack;     /**< Funcin de retrollamada que se ejecutar al completarse el temporizador */
    SDL_TimerID id;                 /**< Identificador del temporizador */
    Uint32 interval;                /**< Intervalo de tiempo a esperar por el temporizador */
    void *param;                    /**< Parmetro a pasar a la funcin de retrollamada */

public:
    /** \brief Constructor por defecto.
     *
     */
    EXPORT Timer() : CallBack(0), id(0), interval(0), param(NULL) {}
    /** \brief Constructor
     *
     * \param i Uint32 Intervalo de tiempo a esperar por el temporizador
     * \param p void* Parmetro a enviar a la funcin de retrollamada
     *
     */
    EXPORT Timer(Uint32 i, void *p) : interval(i), param(p) {}
    /** \brief Destructor
     *
     * Destruye el temporizador.
     *
     */
    EXPORT ~Timer() {
        if(id!=0) SDL_RemoveTimer(id);
    }
    /** \brief Asigna una nueva funcin de retrollamada al temporizador.
     *
     * \param callback SDL_TimerCallback Nueva funcin de retrollamada
     *
     */
    EXPORT void SetCallback(SDL_TimerCallback callback) {
        CallBack = callback;
    }
    /** \brief Asigna un nuevo intervalo de espera.
     *
     * \param i Uint32 Nuevo valor del intervalo
     * \return Uint32 Valor previo del intervalo
     *
     */
    EXPORT Uint32 SetInterval(Uint32 i) {
        Uint32 retval = interval;
        interval = i;
        return retval;
    }
    /** \brief Asigna el puntero de datos a pasar a la funcin de retrollamada.
     *
     * \param p void* Nuevos datos
     * \return void* Valor previo del puntero de datos
     *
     */
    EXPORT void* SetParam(void *p) {
        void* retval = param;
        param = p;
        return retval;
    }
    /** \brief Inicia el temporizador
     *
     * \return SDL_TimerID Identificador del temporizador
     *
     */
    EXPORT SDL_TimerID Init() {
        id = SDL_AddTimer(interval, CallBack, param);
        return id;
    }
    /** \brief Obtiene el identificador del temporizador
     *
     * \return SDL_TimerID Identificador
     *
     */
    EXPORT SDL_TimerID Id() const { return id; }
};

} // namespace

#endif // _SDL_TIMER
