/*
 * Clases para manejar ventanas en SDL2
 * SDL_CreateWindow
 * SDL_DestroyWindow
 * SDL_GetWindowSize
 */
#ifndef _SDL_WINDOW
#define _SDL_WINDOW

#define EXPORT __declspec(dllexport)

#include <SDL.h>
#include <string>

namespace sdl {

/** \class Window
 * \brief Clase que encapsula una ventana SDL2.
 *
 */
class Window {
protected:
    SDL_Window* window = nullptr; /**< Manipulador de la ventana. */

public:
    /** \brief Constructor por defecto.
     *
     */
    EXPORT Window() : window(NULL) {}
    /** \brief Constructor principal. Crea una ventana centrada con el título, dimensiones y banderas especificadas.
     *
     * \param titulo string& Título de la ventana.
     * \param w int Anchura en pixels.
     * \param h int Altura en pixels.
     * \param flags SDL_WindowFlags Banderas opcionales, ver SDL_WindowFlags.
     *
     */
    EXPORT Window(const std::string &titulo, int w, int h, SDL_WindowFlags flags) {
        window = SDL_CreateWindow(titulo.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                w, h, flags);
    }
    /** \brief
     * Destructor.
     *
     */
    EXPORT ~Window() {
        SDL_DestroyWindow(window);
    }
    /** \brief Devuelve el manipulador de la ventana.
     *
     * \return SDL_Window* El manipulador de la ventana.
     *
     */
    EXPORT SDL_Window* Get() const { return window; }
    /** \brief Devuelve el manipulador de la ventana.
     *
     * \return SDL_Window* El manipulador de la ventana.
     *
     */
    EXPORT operator SDL_Window*() { return window; }
    /** \brief Obtiene el tamaño de la ventana.
     *
     * \param w int* Puntero a int que contendrá la anchura en pixels al retornar la función.
     * \param h int* Puntero a int que contendrá la altura en pixels al retornar la función.
     *
     */
    EXPORT void GetSize(int *w, int *h) {
        SDL_GetWindowSize(window, w, h);
    }
    /** \brief Asigna el tamaño de la ventana.
     *
     * \param w int int que contiene la nueva anchura en pixels de la ventana
     * \param h int int que contiene la nueva altura en pixels de la ventana.
     *
     */
    EXPORT void SetSize(int w, int h) {
        SDL_SetWindowSize(window, w, h);
    }
    /** \brief
     * Centra la ventana.
     *
     */
    EXPORT void Center() {
        SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
    }
};

} // namespace

#endif // _SDL_WINDOW
